/* Program     : pcfm
   Version     : 1.10
   Description : Command line controller for Gemtek PCFM serial line 
		 FM radio.  All code based on, or written as original 
		 pcradio code by Chris Radek. Additions for AM/FM and
		 additional hardware by Atsushi Umeshima.
                 User should create a symbolic link /dev/radio
                 with whatever serial port you intend to use.
   Author      : Copyright (C) 1997 Stew Benedict (benedict@netcom.com)
                 1515 Winchester
                 Lakewood, OH 44107

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "pcfm.h"
int fd, c, i;
int mixerfd;
double freq;
int ifreq, ivolume;
int mute = 0;
char freqstr[20];
char volstr[20];
char char_buffer[255];
char *station, *volume;
extern char *optarg;

#define VERSION         "v1.10"

void usage( char *name )
{
  printf( "\n%s "VERSION" "
         "by Stew Benedict (benedict@netcom.com) "
         "designed to be used with the\n"
         "Gemtek PCFM radio attached to a serial port. "
	 "Use a symbolic link to /dev/radio.\n"
	 "All code based on or reused from original" 
	 " pcradio code by Chris Radek\n(cradek@hoser.inetnebr.com)\n"
	 "Additions for AM/FM and additional hardware"
	 " by Atsushi Umeshima\n\n"
	 "In pipeline mode, the same switches apply, without the '-' "
	 "Type 'q' to exit\n"
         "\t\nUsage:\n"
         "\t%s:   -s 'FM station to tune, ie 89.3' \n"
         "\t        -c 'AM station to tune, ie 1134' \n"
         "\t        -n ON \n"
	 "\t        -f OFF \n"
	 "\t        -v 'volume 0-100' \n"
	 "\t        -p pipeline mode for xpcfm \n"
         "\t        -h list usage \n"
         "\t        -? list usage \n",name, name); 
}

void exitpcradio() {
  radiooff(fd);
  radioclose(fd);
  closemixer(mixerfd);
  exit(0);
}

void dopipeline( char *name ) {
        
    while ( fgets ( char_buffer, 20, stdin )){
            
        switch (char_buffer[0]) {
	    case 'n':
	     	radioon(fd);
		break;
	    case 'f':
		radiooff(fd);
		break;
	    case 's':
		*freqstr = '\0';
		for ( i=1; i<=strlen(char_buffer); i++ ) {
		  freqstr[i-1] = char_buffer[i];
	        }     
                freq = atof(freqstr);
  	        ifreq = (int)(((200.0 * freq - 15200.0)/10.0) + 0.5);
  	        sprintf(freqstr, "%03d", ifreq);
  	        radiotune(fd, freqstr, FM);
		break;
	    case 'c':
		*freqstr = '\0';
		for ( i=1; i<=strlen(char_buffer); i++ ) {
		  freqstr[i-1] = char_buffer[i];
	        }     
		freq = atof(freqstr);
		ifreq = (int)((freq - 531.0)/9.0);
  	        sprintf(freqstr, "%03d", ifreq);
  	        radiotune(fd, freqstr, AM);
		break;
	    case 'v':
		*volstr = '\0';
		for ( i=1; i<=strlen(char_buffer); i++ ) {
		   volstr[i-1] = char_buffer[i];
	        }     
                ivolume = atoi(volstr);
                setvolume(mixerfd, ivolume);
                break;
	    case 'q':
		exitpcradio();
            default:
		usage( name );
		break;
	 }
    }
}

int main(int argc, char **argv) {

/* if no command line args, give usage and exit */

  if ( argc == 1 ) { 
      usage( argv[0] );
      exit(0);
  }

  mixerfd = openmixer();
  fd = radioinit();
  setvolume(mixerfd, mute);
  radioon(fd);

  signal(SIGINT, exitpcradio);
  signal(SIGTERM, exitpcradio);

/* process the command line options and decide what to do */
  
  while( ( c = getopt( argc, argv, "s:c:nfv:ph?" ) ) != -1 ) {

    switch( c ) {

        case 's':
          station = optarg;
	  freq = atof(station);
  	  ifreq = (int)(((200.0 * freq - 15200.0)/10.0) + 0.5);
  	  sprintf(freqstr, "%03d", ifreq);
  	  radiotune(fd, freqstr, FM);
          break;
        case 'c':
          station = optarg;
	  freq = atof(station);
	  ifreq = (int)((freq - 531.0)/9.0);
  	  sprintf(freqstr, "%03d", ifreq);
  	  radiotune(fd, freqstr, AM);
          break;
	case 'n':
	  break;
        case 'f':
          exitpcradio();
	  break;
	case 'v':
	  volume = optarg;
          ivolume = atoi(volume);
          setvolume(mixerfd, ivolume);
	  break;
        case 'p':
          dopipeline( argv[0] );
        case 'h':
        case '?':
        default:
          usage( argv[0] );
          exitpcradio();
    }
  }

/* just hang around until the user kills the process */
          
pause();  
}

